package cn.newfeifan.mall.module.distri.service.ptprofit;

import cn.newfeifan.mall.framework.common.pojo.PageResult;
import cn.newfeifan.mall.framework.common.util.object.BeanUtils;
import cn.newfeifan.mall.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.newfeifan.mall.module.distri.controller.admin.ptprofit.vo.PtProfitPageReqVO;
import cn.newfeifan.mall.module.distri.controller.admin.ptprofit.vo.PtProfitSaveReqVO;
import cn.newfeifan.mall.module.distri.dal.dataobject.ptprofit.PtProfitDO;
import cn.newfeifan.mall.module.distri.dal.mysql.ptprofit.PtProfitMapper;
import cn.newfeifan.mall.module.distri.service.ptprofitlog.PtProfitLogService;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.List;

import static cn.newfeifan.mall.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.newfeifan.mall.module.distri.enums.ErrorCodeConstants.PT_PROFIT_NOT_EXISTS;

/**
 * 平台利润 Service 实现类
 *
 * @author 非繁人
 */
@Service
@Validated
public class PtProfitServiceImpl implements PtProfitService {

    @Resource
    private PtProfitMapper ptProfitMapper;

    @Resource
    private PtProfitLogService ptProfitLogService;

    @Override
    public Long createPtProfit(PtProfitSaveReqVO createReqVO) {
        // 插入
        PtProfitDO ptProfit = BeanUtils.toBean(createReqVO, PtProfitDO.class);
        ptProfitMapper.insert(ptProfit);
        // 返回
        return ptProfit.getId();
    }

    @Override
    public void updatePtProfit(PtProfitSaveReqVO updateReqVO) {
        // 校验存在
        validatePtProfitExists(updateReqVO.getId());
        // 更新
        PtProfitDO updateObj = BeanUtils.toBean(updateReqVO, PtProfitDO.class);
        ptProfitMapper.updateById(updateObj);
    }

    @Override
    public void deletePtProfit(Long id) {
        // 校验存在
        validatePtProfitExists(id);
        // 删除
        ptProfitMapper.deleteById(id);
    }

    private void validatePtProfitExists(Long id) {
        if (ptProfitMapper.selectById(id) == null) {
            throw exception(PT_PROFIT_NOT_EXISTS);
        }
    }

    @Override
    public PtProfitDO getPtProfit(Long id) {
        return ptProfitMapper.selectById(id);
    }

    @Override
    public PageResult<PtProfitDO> getPtProfitPage(PtProfitPageReqVO pageReqVO) {
        return ptProfitMapper.selectPage(pageReqVO);
    }

    @Override
    public PtProfitDO getPtProfit() {
        return ptProfitMapper.selectList(new LambdaQueryWrapperX<PtProfitDO>()).get(0);
    }

    @Override
    public void saveBatch(List<PtProfitSaveReqVO> ptProfitSaveReqVOS) {

        List<PtProfitDO> ptProfit = BeanUtils.toBean(ptProfitSaveReqVOS, PtProfitDO.class);
        ptProfitMapper.insertBatch(ptProfit);

    }

}