package cn.newfeifan.mall.module.distri.service.sharepath;

import java.util.*;
import javax.validation.*;

import cn.newfeifan.mall.framework.common.pojo.PageParam;
import cn.newfeifan.mall.module.distri.controller.app.sharepath.vo.*;
import cn.newfeifan.mall.module.distri.dal.dataobject.sharepath.SharePathDO;
import cn.newfeifan.mall.framework.common.pojo.PageResult;

/**
 * 分销用户关系 Service 接口
 *
 * @author 非繁人
 */
public interface SharePathService {

    /**
     * 创建分销用户关系
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createSharePath(@Valid SharePathSaveReqVO createReqVO);

    void createSharePathByUserId(SharePathSaveReqVO createReqVO);
    void createSharePathByUserId(Long userId);



    /**
     * 更新分销用户关系
     *
     * @param updateReqVO 更新信息
     */
    void updateSharePath(@Valid SharePathSaveReqVO updateReqVO);

    /**
     * 删除分销用户关系
     *
     * @param id 编号
     */
    void deleteSharePath(Long id);

    /**
     * 获得分销用户关系
     *
     * @param id 编号
     * @return 分销用户关系
     */
    SharePathDO getSharePath(Long id);

    /**
     * 获得分销用户关系分页
     *
     * @param pageReqVO 分页查询
     * @return 分销用户关系分页
     */
    PageResult<SharePathDO> getSharePathPage(SharePathPageReqVO pageReqVO);


    TreeNode getTree();


    /**
     * 转让分销用户关系
     * @param sharePathtransFerenceRelaReqVO 转让关系
     */
    void transferenceRela(SharePathtransFerenceRelaReqVO sharePathtransFerenceRelaReqVO);


    List<Long> sonsId(Long userId);

    Boolean hasParent(Long userId);

    AppSharePathRespVO getTeam(PageParam pageParam);

    /**
     * 通过直推人, 获取当前的直推关系
     *
     * @param userId 用户id
     * @return 推荐人
     */
    SharePathDO getSharePathByDescendant(Long userId);
}