package cn.newfeifan.mall.module.distri.service.socialstatus;

import javax.validation.*;

import cn.newfeifan.mall.module.distri.controller.admin.socialstatus.bo.SocialStatusBO;
import cn.newfeifan.mall.module.distri.controller.admin.socialstatus.vo.*;
import cn.newfeifan.mall.module.distri.dal.dataobject.socialstatus.SocialStatusDO;
import cn.newfeifan.mall.framework.common.pojo.PageResult;

/**
 * 分销用户身价 Service 接口
 *
 * @author 非繁人
 */
public interface SocialStatusService {

    /**
     * 创建分销用户身价
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createSocialStatus(@Valid SocialStatusSaveReqVO createReqVO);

    /**
     * 更新分销用户身价
     *
     * @param updateReqVO 更新信息
     */
    void updateSocialStatus(@Valid SocialStatusSaveReqVO updateReqVO);

    /**
     * 删除分销用户身价
     *
     * @param id 编号
     */
    void deleteSocialStatus(Long id);

    /**
     * 获得分销用户身价
     *
     * @param id 编号
     * @return 分销用户身价
     */
    SocialStatusDO getSocialStatus(Long id);

    /**
     * 获得分销用户身价分页
     *
     * @param pageReqVO 分页查询
     * @return 分销用户身价分页
     */
    PageResult<SocialStatusDO> getSocialStatusPage(SocialStatusPageReqVO pageReqVO);


    /**
     * 获得身价对象
     *
     * @param point 用户当前身价
     * @return 身价对象
     */
    SocialStatusBO calculateSocial(Long point);


    /**
     * 获取身价等级名称
     * @param socialStatusId 身价编号
     * @return 等级名称
     */
    String getSocialStatusName(Long socialStatusId);
}