package cn.newfeifan.mall.module.sale.service.merchantapplylog;

import cn.newfeifan.mall.module.sale.controller.app.merchantapplylog.vo.MerchantApplyLogPageReqVO;
import cn.newfeifan.mall.module.sale.controller.app.merchantapplylog.vo.MerchantApplyLogSaveReqVO;
import org.springframework.stereotype.Service;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;

import cn.newfeifan.mall.module.sale.dal.dataobject.merchantapplylog.MerchantApplyLogDO;
import cn.newfeifan.mall.framework.common.pojo.PageResult;
import cn.newfeifan.mall.framework.common.util.object.BeanUtils;

import cn.newfeifan.mall.module.sale.dal.mysql.merchantapplylog.MerchantApplyLogMapper;

import static cn.newfeifan.mall.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.newfeifan.mall.module.sale.enums.ErrorCodeConstants.MERCHANT_APPLY_LOG_NOT_EXISTS;

/**
 * 商户审核记录 Service 实现类
 *
 * @author 非繁人
 */
@Service
@Validated
public class MerchantApplyLogServiceImpl implements MerchantApplyLogService {

    @Resource
    private MerchantApplyLogMapper merchantApplyLogMapper;

    @Override
    public Long createMerchantApplyLog(MerchantApplyLogSaveReqVO createReqVO) {
        // 插入
        MerchantApplyLogDO merchantApplyLog = BeanUtils.toBean(createReqVO, MerchantApplyLogDO.class);
        merchantApplyLogMapper.insert(merchantApplyLog);
        // 返回
        return merchantApplyLog.getId();
    }

    @Override
    public void updateMerchantApplyLog(MerchantApplyLogSaveReqVO updateReqVO) {
        // 校验存在
        validateMerchantApplyLogExists(updateReqVO.getId());
        // 更新
        MerchantApplyLogDO updateObj = BeanUtils.toBean(updateReqVO, MerchantApplyLogDO.class);
        merchantApplyLogMapper.updateById(updateObj);
    }

    @Override
    public void deleteMerchantApplyLog(Long id) {
        // 校验存在
        validateMerchantApplyLogExists(id);
        // 删除
        merchantApplyLogMapper.deleteById(id);
    }

    private void validateMerchantApplyLogExists(Long id) {
        if (merchantApplyLogMapper.selectById(id) == null) {
            throw exception(MERCHANT_APPLY_LOG_NOT_EXISTS);
        }
    }

    @Override
    public MerchantApplyLogDO getMerchantApplyLog(Long id) {
        return merchantApplyLogMapper.selectById(id);
    }

    @Override
    public PageResult<MerchantApplyLogDO> getMerchantApplyLogPage(MerchantApplyLogPageReqVO pageReqVO) {
        return merchantApplyLogMapper.selectPage(pageReqVO);
    }

}